/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2023,2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "obj_any.h"

#ifndef OSM_OBJ_NODE_H
#define OSM_OBJ_NODE_H

#include "tags.h"


typedef struct osm_node_data_s {
	osm_latlon_t lon, lat;
	osm_tags_t tags;
} osm_node_data_t;

struct osm_node_s {
	osm_hdr_t hdr;
	osm_node_data_t *curr;
	osm_node_data_t orig;
	/* cached from curr */
	rnd_coord_t x, y; /* screen coords - updated when bbox is calculated from curr */
	rnd_coord_t att_x, att_y; /* screen coords when drawing attached */
};

void osm_node_init(osm_node_t *node);
osm_node_t *osm_node_new(void);


/*** Utility ***/
void osm_node_update_referee_node_att(osm_any_obj_t *o);

/* Return if a node is orphaned and can be removed; if ignore_referer is not
   NULL, don't count that referer */
int osm_node_is_orphan(osm_node_t *node, osm_any_obj_t *ignore_referer);


/* undoable node edit: move node to attached coords */
void osm_node_move_to_att(osm_node_t *node, int undoable);

/* standard undoable mod */
void osm_node_modify(osm_layer_t *layer, osm_node_t *node, double *lon, double *lat, int undoable);
void osm_node_modify_fork(osm_layer_t *layer, osm_node_t *node, int undoable);


/*** for internal use ***/
void osm_node_pre_edit(osm_any_obj_t *o, int geo, int auto_fork);
void osm_node_post_edit(osm_any_obj_t *o, int tags_changed);
void osm_node_referee_nodes(osm_any_obj_t *o, vtp0_t *dst, int update_att);

#endif
