/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2023,2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "obj_any.h"

#ifndef OSM_OBJ_RELATION_H
#define OSM_OBJ_RELATION_H

#include "config.h"

#include "tags.h"

#include "vtr0.h"

typedef struct osm_relation_data_s {
	vtr0_t members;     /* vector of osm_relmember_t */
	osm_tags_t tags;
} osm_relation_data_t;


typedef struct osm_relation_s {
	osm_hdr_t hdr;
	osm_relation_data_t *curr;
	osm_relation_data_t orig;
} osm_relation_t;

void osm_relation_init(osm_relation_t *relation);
osm_relation_t *osm_relation_new(void);


/*** Utility ***/

/* Returns 1 if obj is used by relation */
int osm_relation_refers_obj(osm_relation_t *rel, osm_any_obj_t *obj);

/* List and mark relations that refer to obj */
long osm_relation_list_referers_by_obj(vtp0_t *lst, osm_any_obj_t *obj);

/* Mark relations dirty if they refer obj */
long osm_relation_dirt_referers_by_obj(osm_any_obj_t *obj);

#endif
